@@ -17,7 +17,7 @@ instance as a gRPC server so that it can be used as a Pulumi plugin.
 
 """
 
-from typing import Dict, List, Set, Optional, TypeVar, Any
+from typing import Dict, List, Set, Optional, TypeVar, Any, cast
 import argparse
 import asyncio
 import sys
@@ -25,10 +25,9 @@ import sys
 import grpc
 import grpc.aio
 
-from pulumi._async import _asynchronized
-from pulumi.provider.provider import Provider, ConstructResult
+from pulumi.provider.provider import Provider, CallResult, ConstructResult
 from pulumi.resource import Resource, DependencyResource, DependencyProviderResource
-from pulumi.runtime import proto, rpc
+from pulumi.runtime import known_types, proto, rpc
 from pulumi.runtime.proto import provider_pb2_grpc, ResourceProviderServicer
 from pulumi.runtime.stack import wait_for_rpcs
 import pulumi
@@ -55,10 +54,20 @@ class ProviderServicer(ResourceProviderServicer):
     engine_address: str
     provider: Provider
     args: List[str]
+    lock: asyncio.Lock
+
 
-    # NOTE: remove @_asynchronized when we can avoid modifying globals in the method body.
-    @_asynchronized
     async def Construct(self, request: proto.ConstructRequest, context) -> proto.ConstructResponse:  # pylint: disable=invalid-overridden-method
+        # Calls to `Construct` and `Call` are serialized because they currently modify globals. When we are able to
+        # avoid modifying globals, we can remove the locking.
+        await self.lock.acquire()
+        try:
+            return await self._construct(request, context)
+        finally:
+            self.lock.release()
+
+    async def _construct(self, request: proto.ConstructRequest, context) -> proto.ConstructResponse:
+        # pylint: disable=unused-argument
         assert isinstance(request, proto.ConstructRequest), \
             f'request is not ConstructRequest but is {type(request)} instead'
 
@@ -72,7 +81,7 @@ class ProviderServicer(ResourceProviderServicer):
 
         pulumi.runtime.config.set_all_config(dict(request.config), request.configSecretKeys)
 
-        inputs = self._construct_inputs(request)
+        inputs = await self._construct_inputs(request)
 
         result = self.provider.construct(name=request.name,
                                          resource_type=request.type,
@@ -92,28 +101,28 @@ class ProviderServicer(ResourceProviderServicer):
         return response
 
     @staticmethod
-    def _construct_inputs(request: proto.ConstructRequest) -> Dict[str, pulumi.Output]:
+    async def _construct_inputs(request: proto.ConstructRequest) -> Dict[str, pulumi.Input[Any]]:
 
-        def deps(key: str) -> Set[Resource]:
-            return set(DependencyResource(urn) for urn in
+        def deps(key: str) -> Set[str]:
+            return set(urn for urn in
                        request.inputDependencies.get(
                            key,
                            proto.ConstructRequest.PropertyDependencies()
                        ).urns)
 
         return {
-            k: ProviderServicer._construct_output(the_input, deps=deps(k))
+            k: await ProviderServicer._create_output(the_input, deps=deps(k))
             for k, the_input in
             rpc.deserialize_properties(request.inputs, keep_unknowns=True).items()
         }
 
     @staticmethod
-    def _construct_output(the_input: Any, deps: Set[Resource]) -> Any:
+    async def _create_output(the_input: Any, deps: Set[str]) -> Any:
         is_secret = rpc.is_rpc_secret(the_input)
 
-        # If it's a prompt value, return it directly without wrapping
+        # If it's a resource reference or a prompt value, return it directly without wrapping
         # it as an output.
-        if not is_secret and len(deps) == 0:
+        if await _is_resource_reference(the_input, deps) or (not is_secret and len(deps) == 0):
             return the_input
 
         # Otherwise, wrap it as an output so we can handle secrets
@@ -121,7 +130,7 @@ class ProviderServicer(ResourceProviderServicer):
         # Note: If the value is or contains an unknown value, the Output will mark its value as
         # unknown automatically, so we just pass true for is_known here.
         return pulumi.Output(
-            resources=deps,
+            resources=set(DependencyResource(urn) for urn in deps),
             future=_as_future(rpc.unwrap_rpc_secret(the_input)),
             is_known=_as_future(True),
             is_secret=_as_future(is_secret))
@@ -158,6 +167,86 @@ class ProviderServicer(ResourceProviderServicer):
                                        state=state,
                                        stateDependencies=deps)
 
+
+    async def Call(self, request: proto.CallRequest, context):  # pylint: disable=invalid-overridden-method
+        # Calls to `Construct` and `Call` are serialized because they currently modify globals. When we are able to
+        # avoid modifying globals, we can remove the locking.
+        await self.lock.acquire()
+        try:
+            return await self._call(request, context)
+        finally:
+            self.lock.release()
+
+    async def _call(self, request: proto.CallRequest, context):
+        # pylint: disable=unused-argument
+        assert isinstance(request, proto.CallRequest), \
+            f'request is not CallRequest but is {type(request)} instead'
+
+        pulumi.runtime.settings.reset_options(
+            project=_empty_as_none(request.project),
+            stack=_empty_as_none(request.stack),
+            parallel=_zero_as_none(request.parallel),
+            engine_address=self.engine_address,
+            monitor_address=_empty_as_none(request.monitorEndpoint),
+            preview=request.dryRun)
+
+        pulumi.runtime.config.set_all_config(dict(request.config), request.configSecretKeys)
+
+        args = await self._call_args(request)
+
+        result = self.provider.call(token=request.tok, args=args)
+
+        response = await self._call_response(result)
+
+        # Wait for outstanding RPCs such as more provider Construct
+        # calls. This can happen if i.e. provider creates child
+        # resources but does not await their URN promises.
+        #
+        # Do not await all tasks as that starts hanging waiting for
+        # indefinite grpc.aio servier tasks.
+        await wait_for_rpcs(await_all_outstanding_tasks=False)
+
+        return response
+
+    @staticmethod
+    async def _call_args(request: proto.CallRequest) -> Dict[str, pulumi.Input[Any]]:
+
+        def deps(key: str) -> Set[str]:
+            return set(urn for urn in
+                       request.argDependencies.get(
+                           key,
+                           proto.CallRequest.ArgumentDependencies()
+                       ).urns)
+
+        return {
+            k: await ProviderServicer._create_output(the_input, deps=deps(k))
+            for k, the_input in
+            # We need to keep_internal, to keep the `__self__` that would normally be filtered because
+            # it starts with "__".
+            rpc.deserialize_properties(request.args, keep_unknowns=True, keep_internal=True).items()
+        }
+
+    async def _call_response(self, result: CallResult):
+        # Note: ret_deps is populated by rpc.serialize_properties.
+        ret_deps: Dict[str, List[pulumi.resource.Resource]] = {}
+        ret = await rpc.serialize_properties(
+            inputs=result.outputs,
+            property_deps=ret_deps)
+
+        deps: Dict[str, proto.CallResponse.ReturnDependencies] = {}
+        for k, resources in ret_deps.items():
+            urns = await asyncio.gather(*(r.urn.future() for r in resources))
+            deps[k] = proto.CallResponse.ReturnDependencies(urns=urns)
+
+        # Since `return` is a keyword, we need to pass the args to `CallResponse` using a dictionary.
+        resp = {
+            'return': ret,
+            'returnDependencies': deps,
+        }
+        if result.failures:
+            resp['failures'] = [proto.CheckFailure(property=f.property, reason=f.reason) for f in result.failures]
+        return proto.CallResponse(**resp)
+
     async def Configure(self, request, context) -> proto.ConfigureResponse:  # pylint: disable=invalid-overridden-method
         return proto.ConfigureResponse(acceptSecrets=True, acceptResources=True)
 
@@ -175,6 +264,7 @@ class ProviderServicer(ResourceProviderServicer):
         self.provider = provider
         self.args = args
         self.engine_address = engine_address
+        self.lock = asyncio.Lock()
 
 
 def main(provider: Provider, args: List[str]) -> None:  # args not in use?
@@ -229,3 +319,12 @@ def _empty_as_none(text: str) -> Optional[str]:
 
 def _zero_as_none(value: int) -> Optional[int]:
     return None if value == 0 else value
+
+
+async def _is_resource_reference(the_input: Any, deps: Set[str]) -> bool:
+    """
+    Returns True if `the_input` is a Resource and only depends on itself.
+    """
+    return (known_types.is_resource(the_input)
+        and len(deps) == 1
+        and next(iter(deps)) == await cast(Resource, the_input).urn.future())
