@@ -586,10 +586,15 @@ func TestConstructUnknownGo(t *testing.T) {
 func TestConstructMethodsGo(t *testing.T) {
 	tests := []struct {
 		componentDir string
+		env          []string
 	}{
 		{
 			componentDir: "testcomponent",
 		},
+		{
+			componentDir: "testcomponent-python",
+			env:          []string{pulumiRuntimeVirtualEnv(t, filepath.Join("..", ".."))},
+		},
 		{
 			componentDir: "testcomponent-go",
 		},
@@ -598,7 +603,7 @@ func TestConstructMethodsGo(t *testing.T) {
 		t.Run(test.componentDir, func(t *testing.T) {
 			pathEnv := pathEnv(t, filepath.Join("construct_component_methods", test.componentDir))
 			integration.ProgramTest(t, &integration.ProgramTestOptions{
-				Env: []string{pathEnv},
+				Env: append(test.env, pathEnv),
 				Dir: filepath.Join("construct_component_methods", "go"),
 				Dependencies: []string{
 					"github.com/pulumi/pulumi/sdk/v3",
