@@ -701,10 +701,15 @@ func TestConstructUnknownPython(t *testing.T) {
 func TestConstructMethodsPython(t *testing.T) {
 	tests := []struct {
 		componentDir string
+		env          []string
 	}{
 		{
 			componentDir: "testcomponent",
 		},
+		{
+			componentDir: "testcomponent-python",
+			env:          []string{pulumiRuntimeVirtualEnv(t, filepath.Join("..", ".."))},
+		},
 		{
 			componentDir: "testcomponent-go",
 		},
@@ -713,7 +718,7 @@ func TestConstructMethodsPython(t *testing.T) {
 		t.Run(test.componentDir, func(t *testing.T) {
 			pathEnv := pathEnv(t, filepath.Join("construct_component_methods", test.componentDir))
 			integration.ProgramTest(t, &integration.ProgramTestOptions{
-				Env: []string{pathEnv},
+				Env: append(test.env, pathEnv),
 				Dir: filepath.Join("construct_component_methods", "python"),
 				Dependencies: []string{
 					filepath.Join("..", "..", "sdk", "python", "env", "src"),
